program MicrosAp;

{-----------------------------------------------------------------------
Example application for the MicroScroll (MICROS.DLL) of the SMMWCC 1.0
Custom Control Pack

Copyright (C) Sebastian Modersohn
-----------------------------------------------------------------------}

{$R MicrosAp}

uses WinTypes, WinProcs, Win31, Objects, OWindows, ODialogs, BWCC,
     {Interface unit of the control}
     Micro, MicroCo;

{for loading the DLL}
const DLLName        ='MICROS.DLL';
      em_DLLNotFound =1;

type
  PMicroWindow = ^TMicroWindow;
  TMicroWindow = object(TDlgWindow)
    AEC: PMicroEC;
    MyMicro: PMicro;
    constructor Init(AParent: PWindowsObject; AName: PChar);

    procedure Ok(var Msg: TMessage);
      virtual id_First + id_OK;
    procedure Enable(var Msg: TMessage);
      virtual id_First + 301;
    procedure Disable(var Msg: TMessage);
      virtual id_First + 302;
    procedure Help(var Msg: TMessage);
      virtual id_First + idHelp;
  end;

  {App that loads and frees the DLL and initializes the main window}
  PMicroApp = ^TMicroApp;
  TMicroApp = object(TApplication)
    Lib: THandle;
    constructor Init(AName: PChar);
    destructor Done; virtual;
    procedure InitMainWindow; virtual;
    procedure Error(ErrorCode: Integer); virtual;
  end;

{ TMicroWindow }

constructor TMicroWindow.Init(AParent: PWindowsObject; AName: PChar);
begin
  inherited Init(AParent, AName);
  {Init a PEdit descendant which is corresponding with the micro
  Id: 201, Micro: nil, From: 0, To: 10, Step: 1, Start: 4, Unit: inch}
  AEC:=New(PMicroEC, InitResource(@Self, 201, nil, 0, 10, 1, 4, ' inch'));

  {Init a MicroScroll which is corresponding with the Edit (AEC)
  Id: 202, Edit: AEC!!}
  MyMicro:=New(PMicro, InitResource(@Self, 202, AEC));

  {Tell the Edit (AEC) which MicroScroll is corresponding with it}
  AEC^.Micro:=MyMicro;
end;

procedure TMicroWindow.Ok(var Msg: TMessage);
begin
  CloseWindow;
end;

procedure TMicroWindow.Enable(var Msg: TMessage);
begin
  EnableWindow(GetDlgItem(HWindow, 202), True);
end;

procedure TMicroWindow.Disable(var Msg: TMessage);
begin
  EnableWindow(GetDlgItem(HWindow, 202), False);
end;

procedure TMicroWindow.Help(var Msg: TMessage);
begin
  WinHelp(HWindow, HelpFile, Help_Context, 200);
end;

{ TMicroApp }

constructor TMicroApp.Init(AName: PChar);
begin
  {Load the library}
  Lib := LoadLibrary(DLLName);
  {If it wasn't loaded succesfully, then call error with em_DLLNotFound}
  if Lib < 32 then
    Status := em_DLLNotFound
  else
    inherited Init(AName);
end;

destructor TMicroApp.Done;
begin
  inherited Done;
  {Free the library}
  FreeLibrary(Lib);
end;

procedure TMicroApp.InitMainWindow;
begin
  MainWindow := New(PMicroWindow, Init(nil, MakeIntResource(100)));
end;

procedure TMicroApp.Error(ErrorCode: Integer);
begin
  case ErrorCode of
    em_DLLNotFound:
      begin
        {if the dll could not be loaded display the message and fail the app}
        MessageBox(0, DLLName + ' not found. Please make sure that Windows will found this file!', 'Hard Error',
          mb_Ok or mb_IconStop);
        Halt(ErrorCode);
      end;
  else
    inherited Error(ErrorCode);
  end;
end;

var
  App: TMicroApp;

begin
  App.Init('MicroScroll Demo');
  App.Run;
  App.Done;
end.
